#include <stdbool.h>
#include "processor.h"
#include "fdserial.h"
#include "gcode_struct.h"
#include "simpletools.h"

// cog variables
static int serial_processor_cog = 0;
static int serial_processor_stack[60];

// declare volatiles
volatile bool gcode_struct_received;
volatile GCODE_STRUCT sent_gcode;
volatile char received_struct_array[sizeof(GCODE_STRUCT)];

// declare the serial processing cog function
void serial_processor(void *par);

// start the serial processor
void serial_processor_init()
{
	if(serial_processor_cog == 0)
	{
		serial_processor_cog = 1 + cogstart(&serial_processor, NULL,
			serial_processor_stack, sizeof serial_processor_stack);
	}
}

// stop the serial processor
void serial_processor_stop()
{
	if(serial_processor_cog > 0)
	{
		cogstop(serial_processor_cog - 1);
		serial_processor_cog = 0;
	}
}

// serial processing cog
void serial_processor(void *par)
{
	uint32_t index;
	fdserial *processor;
	char Char;
	bool bEndOfStruct;

	processor = fdserial_open(2, 0, 0, 115200);

	do
	{
		// proceed the struct with a '<' for STX
		// STX = start of text
		// STX can be anything that will not be included in your data 
		// STX must be the same on the transmitting end
		if(fdserial_rxTime(processor, 500) == '<')
		{
			index = 0;
			bEndOfStruct = false;

			memset(&sent_gcode, 0, sizeof(GCODE_STRUCT));
			memset(received_struct_array, 0, sizeof(GCODE_STRUCT));

			while(bEndOfStruct == false)
			{
				Char = fdserial_rxTime(processor, 500);

				// follow the struct with a '>' for ETX
				// ETX = end of text
				// ETX can be anything that will not be included in your data 
				// ETX must be the same on the transmitting end
				if(Char != '>')
				{
					received_struct_array[index] = Char;
					index++;
				}    
				else
				{
					bEndOfStruct = true;
				}
			}

			memcpy(&sent_gcode, &received_struct_array, sizeof(GCODE_STRUCT));

			gcode_struct_received = true;
		}
	}
  
	while(serial_processor_cog > 0);
}